package dev.fitko.fitconnect.sdk.demo;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.model.event.SubmissionStatus;
import dev.fitko.fitconnect.api.domain.model.event.problems.other.TechnicalError;
import dev.fitko.fitconnect.api.domain.model.replychannel.ReplyChannel;
import dev.fitko.fitconnect.api.domain.model.submission.SentSubmission;
import dev.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import dev.fitko.fitconnect.client.RouterClient;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.SubscriberClient;
import dev.fitko.fitconnect.client.bootstrap.ApplicationConfigLoader;
import dev.fitko.fitconnect.client.bootstrap.ClientFactory;
import dev.fitko.fitconnect.client.sender.model.Attachment;
import dev.fitko.fitconnect.client.sender.model.SendableSubmission;
import dev.fitko.fitconnect.client.subscriber.ReceivedSubmission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URI;
import java.nio.file.Path;
import java.util.List;
import java.util.Set;
import java.util.UUID;

public class Demo {

    private static final Logger LOGGER = LoggerFactory.getLogger(Demo.class);

    final SubscriberClient subscriberClient;
    final SenderClient senderClient;
    final RouterClient routerClient;

    public static void main(final String[] args) {

        // config can be loaded by setting env-var FIT_CONNECT_CONFIG or directly via ApplicationConfigLoader.loadConfig("path/to/config.yml")
        final var configPath = Path.of("demo/src/main/resources/config.yml");

        // unique identifier of the destination/Zustellpunkt (https://portal.auth-testing.fit-connect.fitko.dev/destinations)
        final var destinationId = UUID.fromString("DESTINATION_ID");

        final var demo = new Demo(configPath);

        final SentSubmission submissionOne = demo.sendSubmission(destinationId);

        demo.listFirstTenSubmissions(destinationId);
        demo.printStatus(submissionOne);
        demo.getSubmission(submissionOne.getSubmissionId());
        demo.acceptSubmission(submissionOne.getSubmissionId());

        final SentSubmission submissionTwo = demo.sendSubmission(destinationId);
        demo.reject(submissionTwo.getSubmissionId());
        demo.printStatus(submissionTwo);
    }

    Demo(final Path configPath){

        final ApplicationConfig config =  ApplicationConfigLoader.loadConfigFromPath(configPath);

        // init clients
        subscriberClient = ClientFactory.getSubscriberClient(config);
        senderClient = ClientFactory.getSenderClient(config);
        routerClient = ClientFactory.getRouterClient(config);
    }

    private void acceptSubmission(final UUID submissionId) {
        subscriberClient.requestSubmission(submissionId).acceptSubmission();
        LOGGER.info("Accepted submission {}", submissionId);
    }

    private void reject( final UUID submissionId) {
        subscriberClient.requestSubmission(submissionId).rejectSubmission(List.of(new TechnicalError()));
        LOGGER.info("Accepted submission {}", submissionId);
    }

    private void printStatus(final SentSubmission sentSubmission) {
        final SubmissionStatus status = senderClient.getStatusForSubmission(sentSubmission);
        LOGGER.info("Current status for submission {}: {}", sentSubmission.getSubmissionId(), status.getStatus());
    }

    private void getSubmission(final UUID submissionId) {
        final ReceivedSubmission receivedSubmission = subscriberClient.requestSubmission(submissionId);
        LOGGER.info("Received data {}", receivedSubmission.getDataAsString());
        receivedSubmission.getAttachments().stream().map(Attachment::getDataAString).forEach(attachment -> LOGGER.info("Received attachment {}", attachment));

    }

    private void listFirstTenSubmissions(final UUID destinationId) {
        final Set<SubmissionForPickup> availableSubmissions = subscriberClient.getAvailableSubmissionsForDestination(destinationId, 0, 10);
        LOGGER.info("First {} available submission of destination {}", availableSubmissions.size(), destinationId);
        availableSubmissions.forEach(sub -> LOGGER.info("submission-id {}", sub.getSubmissionId()));
    }

    private SentSubmission sendSubmission(final UUID destinationId) {

        final SendableSubmission submission = SendableSubmission.Builder()
                .setDestination(destinationId)
                .setServiceType("urn:de:fim:leika:leistung:99400048079000", "Führerscheinbeantragung")
                .setJsonData( getDataAsString("/data.json"), URI.create("https://schema.fitko.de/fim/s00000114_1.1.schema.json"))
                .addAttachment(Attachment.fromPath(Path.of("demo/src/main/resources/attachment.txt"), "application/text"))
                .setReplyChannel(ReplyChannel.fromEmail("test@sample.org"))
                .build();

        return senderClient.send(submission);
    }

    private String getDataAsString(final String path)  {
        try {
            return new String(Demo.class.getResourceAsStream(path).readAllBytes());
        } catch (final IOException e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }
}
